/* 6545 CRTC Header */

#ifndef HEADER_CRTC_H
#define HEADER_CRTC_H

#include "mz80.h"

#define CRTC_HTOT           0
#define CRTC_HDISP          1
#define CRTC_HSYNC_POS      2
#define CRTC_SYNC_WIDTH     3
#define CRTC_VTOT           4
#define CRTC_VTOT_ADJ       5
#define CRTC_VDISP          6
#define CRTC_VSYNC_POS      7
#define CRTC_MODE           8
#define CRTC_SCANLINES      9
#define CRTC_CUR_START      10
#define CRTC_CUR_END        11
#define CRTC_DISP_START_H   12
#define CRTC_DISP_START_L   13
#define CRTC_CUR_POS_H      14
#define CRTC_CUR_POS_L      15
#define CRTC_LPEN_H         16
#define CRTC_LPEN_L         17
#define CRTC_SETADDR_H      18
#define CRTC_SETADDR_L      19

#define CRTC_DOSETADDR      31

// default video resolution
#define VDUWIDTH  640
#define VDUHEIGHT 275

// Green monitor
#define MONGR_FGR   0
#define MONGR_FGG   210
#define MONGR_FGB   0
#define MONGR_FGR_L 0
#define MONGR_FGG_L 140
#define MONGR_FGB_L 0
#define MONGR_BGR   0
#define MONGR_BGG   0
#define MONGR_BGB   0

// Amber monitor
#define MONAM_FGR   238
#define MONAM_FGG   166
#define MONAM_FGB   0
#define MONAM_FGR_L 158
#define MONAM_FGG_L 110
#define MONAM_FGB_L 0
#define MONAM_BGR   0
#define MONAM_BGG   0
#define MONAM_BGB   0

// White monitor
#define MONWB_FGR   192
#define MONWB_FGG   192
#define MONWB_FGB   192
#define MONWB_FGR_L 128
#define MONWB_FGG_L 128
#define MONWB_FGB_L 128
#define MONWB_BGR   0
#define MONWB_BGG   0
#define MONWB_BGB   0

// Black monitor
#define MONBW_FGR   0
#define MONBW_FGG   0
#define MONBW_FGB_L 0
#define MONBW_FGR_L 0
#define MONBW_FGG_L 0
#define MONBW_FGB   0
#define MONBW_BGR   255
#define MONBW_BGG   255
#define MONBW_BGB   255

// Colour monitor intensity values
// CGA colour values
#define C_FI	252		// colour full intensity 
#define C_HI	168		// colour half intensity
#define C_LI	84		// colour low intensity

extern int disp_start;

void crtc_clock (int xtal, int method);

int crtc_init (void);
int crtc_deinit (void);
int crtc_reset (void);

int crtc_videochange (void);

void crtc_redraw (void);
void crtc_set_redraw (void);
void crtc_redraw_char (int addr, int dostdout);

void crtc_lpen (int addr);

int crtc_vblank (void);
UINT16 crtc_status (UINT16 port, struct z80PortRead *port_struct);
void   crtc_address (UINT16 port, UINT8 data, struct z80PortWrite *port_struct);
UINT16 crtc_data_r (UINT16 port, struct z80PortRead *port_struct);
void   crtc_data_w (UINT16 port, UINT8 data, struct z80PortWrite *port_struct);

void crtc_configure (int aspect);
void crtc_regdump (void);

#endif     /* HEADER_CRTC_H */
