/* function Header */

#ifndef HEADER_FUNCTION_H
#define HEADER_FUNCTION_H

#include <stdint.h>
#include <dirent.h>
#include "mz80.h"

int copy_file (char *dest, char *src);
void function_dump (int addr, int lines, int htype);
void function_emu_command (int cmd);

int16_t little_endian_16 (int16_t n);
uint16_t little_endian_u16 (uint16_t n);
int32_t little_endian_32 (int32_t n);
uint32_t little_endian_u32 (uint32_t n);
int64_t little_endian_64 (int64_t n);
uint64_t little_endian_u64 (uint64_t n);

int16_t big_endian_16 (int16_t n);
uint16_t big_endian_u16 (uint16_t n);
int32_t big_endian_32 (int32_t n);
uint32_t big_endian_u32 (uint32_t n);
int64_t big_endian_64 (int64_t n);
uint64_t big_endian_u64 (uint64_t n);

int function_init (void);
int function_deinit (void);
int function_reset (void);

UINT16 function_ubee_r (UINT16 port, struct z80PortRead *port_s);
void function_ubee_w (UINT16 port, UINT8 data, struct z80PortWrite *port_s);

#define EMU_CMD_FULLSCR   0
#define EMU_CMD_TAPEREW   1
#define EMU_CMD_DUMP      2
#define EMU_CMD_DUMP_N1   3
#define EMU_CMD_DUMP_N2   4
#define EMU_CMD_DUMP_B1   5
#define EMU_CMD_DUMP_B2   6
#define EMU_CMD_DUMP_REP  7
#define EMU_CMD_DUMPREGS  8
#define EMU_CMD_DBGOFF    9
#define EMU_CMD_DBGON     10
#define EMU_CMD_DBGTRACE  11
#define EMU_CMD_DBGSTEP01 12
#define EMU_CMD_DBGSTEP10 13
#define EMU_CMD_DBGSTEP20 14

#pragma pack(push, 1)  // push current alignment, alignment to 1 byte boundary

typedef union fp_t
   {
    uint64_t i;			        // forces 8 bytes of storage
    FILE *p;		                // store a 4 or 8 byte pointer
    DIR *d;
   }fp_t;

// version informnation response structure
typedef struct ub_version_t
   {
    uint16_t cmd;			// sub command
    uint16_t ver1;			// vers 1.x.x value
    uint16_t ver2;			// vers x.1.x value
    uint16_t ver3;			// vers x.x.1 value
   }ub_version_t;

// dump memory structure
typedef struct ub_dump_t
   {
    uint16_t cmd;			// sub command
    uint16_t id;			// 0xAA55 check ID    
    uint16_t addr;			// dump address
    uint16_t lines;			// lines
    uint16_t htype;			// header type
   }ub_dump_t;

// command and response structure
typedef struct ub_cmdres_t
   {
    uint16_t cmd;			// sub command
    uint16_t id;			// 0xAA55 check ID
    int16_t res;			// result
    fp_t fp;
   }ub_cmdres_t;

// file structure
typedef struct ub_file_t
   {
    uint16_t cmd;			// sub command
    uint16_t id;			// 0xAA55 check ID
    int16_t res;			// result
    fp_t fp;
    uint16_t addr1;			// address 1 in Z80 map
    uint16_t addr2;			// address 2 in Z80 map
    uint16_t addr3;			// address 3 in Z80 map
    uint16_t addr4;			// address 4 in Z80 map    
    int16_t size;			// size of object
    uint16_t num;			// number of chars
    int16_t val1;			// value 1
    int16_t val2;			// value 2    
   }ub_file_t;

// stdio input commands
typedef struct ub_stdio_input_t
   {
    uint16_t cmd;			// sub command
    int16_t res;			// result
   }ub_stdio_input_t;

// stdio character write commands
typedef struct ub_stdio_char_t
   {
    uint16_t cmd;			// sub command
    int16_t res;			// result
    uint16_t val;			// value
   }ub_stdio_char_t;

// stdio string commands
typedef struct ub_stdio_str_t
   {
    uint16_t cmd;			// sub command
    int16_t res;			// result
    uint16_t addr;			// address in Z80 map
   }ub_stdio_str_t;

typedef union ub_func_u
   {
    ub_version_t version;
    ub_dump_t dump;
    ub_stdio_input_t getchar;
    ub_stdio_char_t putchar;
    ub_stdio_str_t puts;
    ub_cmdres_t cmdres;
    ub_file_t file;
   }ub_func_u;

#pragma pack(pop)	// restore original alignment from stack

#endif     /* HEADER_FUNCTION_H */
