//******************************************************************************
//*                                  uBee512                                   *
//*         An emulator for the Microbee Z80 FDD and ROM based models          *
//*                                                                            *
//*                             Message Box module                             *
//*                                                                            *
//*                   Copyright (C) 2007-2008 Stewart Kay                      *
//******************************************************************************
//
// Provides message box support for Unices platforms using the Athena widget
// set.
//
//==============================================================================
// ChangeLog (most recent entries are at top)
//==============================================================================
// v2.0.0 - 17 October 2007, S.J.Kay
// - Created a new file and implement the message box.  It operates in the
//   same thread so the SDL window will not update until it is closed.
//   Thanks to M.Kay for providing this code. The structuring is my style so
//   don't blame him.

//============================================================================== 
/*
 *  uBee512 - An emulator for the Microbee Z80 FDD and ROM based models. 
 *  Copyright (C) 2007-2008 Stewart Kay
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//==============================================================================   
#ifndef MINGW

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw3d/Form.h>
#include <X11/Xaw3d/Dialog.h>
#include <X11/Xaw3d/Command.h>

#include "mbox.h"

//==============================================================================
// structures and variables
//==============================================================================
#define MAXBUTTONS 10

static Atom wm_delete_window;
static Display *dpy = NULL;     // connection to the X server
static Screen *screen = NULL;
static Widget toplevel=NULL;  	// top level shell widget
static Widget form=NULL;	// put dialog inside a form to get border
static Widget message=NULL;	// an underlying dialog widget
static Widget buttons[MAXBUTTONS];
static XtAppContext xtcontext;	// Toolkit application context
static int exitmode;
static char exitbutton[20];
static int defaultbutton = 0;	// number of default button
static Pixel pix;

static Arg sethilight = {XtNborderColor, (XtArgVal)NULL};
static Arg setlolight = {XtNborderColor, (XtArgVal)NULL};

//==============================================================================
// actions - externally available procedures for messbox
//
// report button
//
//   pass: Widget w
//         XEvent *e
//         String *vector
//         Cardinal *count
// return: void
//==============================================================================
static void bttn (Widget w, XEvent *e, String *vector, Cardinal *count)
{
 char *any;
 
 Arg label = {XtNlabel, (XtArgVal) & any};

 if (strcmp("?",vector[0]) == 0)
    {
     XtGetValues(buttons[defaultbutton],&label,1);
     strncpy(exitbutton,any,19);
    }
 else
    strncpy(exitbutton,vector[0],19);

 exitbutton[19] = 0;
 XtAppSetExitFlag(xtcontext);
 exitmode = 1;
}

//==============================================================================
// hiho
//
// report unmap
//
//   pass: Widget w
//         XEvent *e
//         String *vector
//         Cardinal *count
// return: void
//==============================================================================
static void hiho (Widget w, XEvent *e, String *vector, Cardinal *count)
{
 if (exitmode == 1)
    {
     XtAppSetExitFlag(xtcontext);
     exitmode = 2;
    }
}

//==============================================================================
// Default
//
//   pass: int it
// return: void
//==============================================================================
static void Default (int it)
{
// Ar arg={XtNbackground, NULL};
 if (it < 0)
    {// If less than button0 go to last button
     for (it=0; (it+1) < MAXBUTTONS && buttons[it+1]; it++)
        ;
    }
 else
    if (! buttons[it])
       it=0;

 defaultbutton = it;

 for (it=0; buttons[it]; it++)
    {
//   printf("set button%d at %p: default is button%d \n", it, buttons[it], defaultbutton);
     if (it == defaultbutton)
        XtSetValues(buttons[it],&sethilight,1);
     else
        XtSetValues(buttons[it],&setlolight,1);
    }
}

//==============================================================================
// dflt
//
// report Tab, Right, Left
//
//   pass: Widget w
//         XEvent *e
//         String *vector
//         Cardinal *count
// return: void
//==============================================================================
static void dflt (Widget w, XEvent *e, String *vector, Cardinal *count)
{
 switch (*vector[0])
    {
     case '-' : Default(defaultbutton-1);
                break;
      default : Default(defaultbutton+1);
    }
}

//==============================================================================
// mbox
//
//   pass: int argc
//         char **argv			pointer to the arguments
//         char *title			pointer for the title name
//         char **buts			pointer to the buttons
//         int defbut			default button on entry
//         
// return: String mbox
//==============================================================================
String mbox (int argc, char **argv, char *title, char **buts, int defbut)
{
 static XtActionsRec Actions[] =
    {
     {"bttn",  bttn},            // report button
     {"hiho",  hiho},            // report unmap
     {"dflt",  dflt},            // report Tab, Right, Left
    };

 static int isdefined = 0;
 
 XWMHints *wmhints;
 XSizeHints *sizehints;

 long supplied_return;
 Dimension height, width;
 
 char geometry[100];
 
 Arg margs[] = 
    {
     { XtNresizable,(XtArgVal)True },
     { XtNinput, False},
     { XtNmappedWhenManaged,False },
     { XtNgeometry,(XtArgVal)geometry },
    };
    
 Arg dargs[] =
    {
     { XtNresizable,(XtArgVal)True },
     { XtNlabel,(XtArgVal)title },
     { XtNtop,(XtArgVal)XawChainTop },
     { XtNbottom,(XtArgVal)XawChainTop },
     { XtNleft,(XtArgVal)XawChainLeft },
     { XtNright,(XtArgVal)XawChainLeft },
  };
    
 Arg gsize[] =
    {
     { XtNheight,(XtArgVal)&height },
     { XtNwidth,(XtArgVal)&width },
    };
    
 Arg gbg[] =
    {
     { XtNbackground,(XtArgVal)&pix },
    };
	
 int i;

 if (! isdefined)
    {
// application shell
     toplevel = XtOpenApplication(&xtcontext, "MessBox", NULL, 0,
     &argc, argv, fallback, sessionShellWidgetClass, (ArgList)NULL, 0);
     dpy = XtDisplay(toplevel);
     screen = XtScreen(toplevel);
// setlolight.value=WhitePixel(dpy,DefaultScreen(dpy));

// place more or less centre of screen
     sprintf(geometry, "+%d+%d", screen->width/2-200, screen->height/2-100);
     XtSetValues(toplevel, margs, 4);
     
// an encompassing form to frame the dialog
     form = XtCreateManagedWidget("form", formWidgetClass, toplevel, (ArgList) dargs, 1);
     
// the actual dialog
     message = XtCreateManagedWidget("message", dialogWidgetClass, form,
     (ArgList) dargs, 6);

// highlight borders for buttons
     sethilight.value = BlackPixel(dpy,DefaultScreen(dpy));

// merge unhighlighted with Dialog background
     XtGetValues(message,gbg,1);
     setlolight.value=pix;

// populate with buttons
     for (i=0; i < MAXBUTTONS; i++)
        {
         char temp1[10];
         char temp2[30];
         
         Arg barg[] =
            {
             { XtNlabel,(XtArgVal)buts[i] },
            };
     
         if (buts[i])
            {
             snprintf(temp1, 10, "button%d", i);
             snprintf(temp2, 30, "<Btn1Down>:bttn(\"%.10s\")", buts[i]);
             buttons[i] = XtCreateManagedWidget(temp1, commandWidgetClass,
             message, (ArgList)barg, 1);
             XtOverrideTranslations(buttons[i], XtParseTranslationTable(temp2));
            }
         else
            break;
        }

// <Return> in Dialog to register as a "Default" button
     XtOverrideTranslations(buttons[0],
     XtParseTranslationTable("<Key>Return:bttn(\"?\")\n"
     "<Key>Tab:dflt(\"+\")\n"
     "<Key>Right:dflt(\"+\")\n"
     "<Key>Left:dflt(\"-\")\n"));
     
     XtSetKeyboardFocus(form, buttons[0]);
     
// unmapping help
     XtOverrideTranslations(toplevel, XtParseTranslationTable("<Unmap>:hiho()\n"));
     
// connect Actions
     XtAppAddActions(xtcontext, Actions, XtNumber(Actions));
     
     XtOverrideTranslations(toplevel,
     XtParseTranslationTable("<Message>WM_PROTOCOLS: bttn(\"delete_window\")\n"));
     XtRealizeWidget(toplevel);

// try to manage size
     XtGetValues(form,gsize,2);
     
//   printf("width=%d; height=%d\n",width,height);
     sizehints = XAllocSizeHints();
     XGetWMNormalHints(dpy, XtWindow(toplevel), sizehints,
     &supplied_return);
     sizehints->flags |= PMinSize | PMaxSize;
     sizehints->min_width = sizehints->max_width=width;
     sizehints->min_height = sizehints->max_height=height;
     XSetWMNormalHints(dpy, XtWindow(toplevel), sizehints);
     wm_delete_window = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
     (void) XSetWMProtocols(dpy, XtWindow(toplevel), &wm_delete_window, 1);

// try set up input focus
     if (!(wmhints = XGetWMHints(dpy, XtWindow(toplevel))))
        wmhints = XAllocWMHints();
     
     wmhints->flags |= InputHint; // add in input hint
     wmhints->input = True;
     XSetWMHints(dpy,XtWindow(toplevel), wmhints);
     XFree((char *) wmhints);
    }
 else
// set new message
    XtSetValues(message,dargs,2);

// highlight default button
 Default(defbut);
 XtAddGrab(message,True,False);
 XtMapWidget(toplevel);

// activate loop looking for a button press
 for (exitmode=0; exitmode != 1; XtAppMainLoop(xtcontext))
    ;

// unmap all
 XtUnmapWidget(toplevel);
   
// and look for <Unmap> event
 for (exitmode=1; exitmode != 2; XtAppMainLoop(xtcontext))
    ;

// return a copy of the bttn() text -- same as button label
 isdefined = 1;

 return exitbutton;
}

#endif
