/* ubee512 Header */

#ifndef HEADER_UBEE512_H
#define HEADER_UBEE512_H

#include "SDL.h"

#ifdef MINGW
#define SLASHCHAR '\\'
#define SLASHCHAR_STR "\\"
#define DIR_DOCS "\\doc\\"
#define DIR_DISKS "\\disks\\"
#define DIR_ROMS "\\roms\\"
#define DIR_TAPES "\\tapes\\"
#define DIR_PRINTER "\\printer\\"
#define DIR_IMAGES "\\images\\"
#define DIR_RTC "\\rtc\\"
#define DIR_TOOLS "\\tools\\"
#else
#define SLASHCHAR '/'
#define SLASHCHAR_STR "/"
#define DIR_DOCS "/doc/"
#define DIR_DISKS "/disks/"
#define DIR_ROMS "/roms/"
#define DIR_TAPES "/tapes/"
#define DIR_PRINTER "/printer/"
#define DIR_IMAGES "/images/"
#define DIR_RTC "/rtc/"
#define DIR_TOOLS "/tools/"
#endif

void event_handler (void);
void z80_exec_ratio (int divider);
void console_output (void);
void set_clock_speed (float clock);

//==============================================================================
// Model property definitions
//==============================================================================
#define MODROM  1
#define MODCOL1 1
#define MODCOL2 2
#define MODSPD  1

#define MODPB7_PUP 0
#define MODPB7_VS 1
#define MODPB7_RTC 2
#define MODPB7_NET 3

#define MODRTC 1

enum
   {
    MOD_256TC,   
    MOD_P512K,
    MOD_512K,
    MOD_P128K,
    MOD_128K,
    MOD_P256K,
    MOD_256K,
    MOD_P64K,
    MOD_64K,
    MOD_56K,
    MOD_PPC85,    
    MOD_PC85,
    MOD_PC,
    MOD_IC,
    MOD_2MHZ,
    MOD_TOTAL
   };

typedef struct model_t
   {
    int alphap;
    int rom;
    int bootaddr;
    int ram;
    int pcg;
    int colour;
    int hwflash;
    int halfint;
    int lpen;
    int speed;
    int piob7;
    int rtc;
    float cpuclock;
   }model_t; 

//==============================================================================
// Debugging related
//==============================================================================
typedef struct debug_t
   {
    char break_point[0x10000];
    int debug;
    int mode;    
    int show;
    int step;
    int debugcount;
    int showcount;
    int break_point_count;    
   }debug_t;

typedef struct modio_t
   {
    FILE *log;
    int level;
   
    int clock;    
    int crtc;
    int fdc;
    int func;
    int keystd;
    int keytc;    
    int mem;
    int paknet;
    int pioa;
    int piob;
    int piocont;
    int raminit;
    int rtc;
    int vdu;
    int vdumem;
    int z80;
   }modio_t;

typedef struct regdump_t
   {
    int clock;
    int crtc;
    int fdc;
    int keytc;    
    int mem;
    int paknet;
    int pio;
    int rtc;
    int vdu;
    int z80;
   }regdump_t;

//==============================================================================
// Compile time options,  these are normally only changed for testing purposes.
//==============================================================================
// paths to shared directories.
#define PATH_SHARED_IMAGES "/usr/local/share/ubee512/images/"
#define PATH_SHARED_DOCS "/usr/local/share/ubee512/doc/"
#define PATH_SHARED_TOOLS "/usr/local/share/ubee512/tools/"
#define PATH_SHARED_DISKS "/usr/local/share/ubee512/disks/"

// default fall-back boot image
#define BOOT_IMAGE "boot.dsk"

// default model to emulate
#define MOD_DEFAULT MOD_P512K

// define this for the default CPU clock frequency (in MHz)
#define	CPU_CLOCK_FREQ 3.375

// defines the target frame rate (frames per second) wanted.
#define FRAMERATE 30

// define this for the default tape out sample frequency (in Hz)
#define	TAPE_SAMPLE_FREQ 22050

// define this for the default tape volume level.
#define	TAPE_VOLUME 16

// define the default serial port values.
#define	SERIAL_RX_BAUD 300
#define	SERIAL_TX_BAUD 300
#define	SERIAL_DATABITS 8
#define	SERIAL_STOPBITS 1

// Sound hold-off delays and speed adjustments.
// It has been found that the Z80 speed increase is no longer required.  If
// sound does break up then try putting the SPEED_PCT_ADJ values back to
// that shown in the commented out line or a lesser amount could also be
// tried.
//
// The sound hold-off values have also been reduced from 100 to 75 mS.
#ifdef MINGW
#define SOUND_HOLD_OFF 75
#define SPEED_PCT_ADJ 0.0	// x% faster to stop audio break up
//#define SPEED_PCT_ADJ 6.0	// x% faster to stop audio break up
#else
#define SOUND_HOLD_OFF 75
#define SPEED_PCT_ADJ 0.0	// x% faster to stop audio break up
//#define SPEED_PCT_ADJ 9.0	// x% faster to stop audio break up
#endif 

// define where the logged data will go
#define LOGFILE	"ubee512_log.txt"

// define the following debug options to report on ports on stdout
#define use_resetinfo
//#define use_debug_disk_read_abort

//#define use_debug_getkeystate_shift
//#define use_debug_getkeystate_lower

// define the following options to be reported to stdout
#define use_info_disk_open
//#define use_info_disk_read

#endif /* HEADER_UBEE512_H */
